[tool.black]
line-length = 100

[tool.pytest.ini_options]
addopts = ["-vvv", "-n", "2", "--log-level", "WARNING", "--color", "yes"]
filterwarnings = ["ignore:AnsibleCollectionFinder has already been configured"]
testpaths = ["tests"]
