---
all:
  hosts:
    server1:
      ansible_host: 192.168.1.101
    server2:
      ansible_host: 192.168.1.102
    switch1:
      ansible_host: 192.168.1.106
    switch2:
      ansible_host: 192.168.1.107
  children:
    web_servers:
      hosts:
        server1:
        server2:
    db_servers:
      hosts:
        server3:
          ansible_host: 192.168.1.103
    switches:
      hosts:
        switch1:
        switch2:
    production:
      hosts:
        server1:
          ansible_host: 192.168.1.101
        server2:
          ansible_host: 192.168.1.102
    test:
      hosts:
        server3:
          ansible_host: 192.168.1.103
