---
# argument spec file for {{ namespace }}.{{ collection_name }}.{{ role_name }}

argument_specs:
  main:
    short_description: Role description.
    options:
      {{ role_name }}_my_variable:
        type: str
        description: A simple string argument for demonstration.
        default: "default_value"
