---
version: 3

images:
  base_image:
    name: quay.io/fedora/fedora:41

dependencies:
  # Use python3
  python_interpreter:
    package_system: python3
    python_path: /usr/bin/python3

  ansible_core:
    package_pip: ansible-core

  ansible_runner:
    package_pip: ansible-runner

  system:
    - openssh-clients
    - sshpass

  python:
    - ansible-navigator
    - boto3
    - requests

  galaxy:
    collections:
      - name: ansible.posix
      - name: ansible.utils

additional_build_steps:
  append_base:
    - RUN $PYCMD -m pip install -U pip

options:
  tags:
    - ansible_sample_ee
