---
# This collection is initialized by https://github.com/ansible/ansible-creator {{ creator_version }}

# See https://docs.ansible.com/ansible/latest/dev_guide/collections_galaxy_meta.html

namespace: "{{ namespace }}"
name: "{{ collection_name }}"
version: 1.0.0
readme: README.md
authors:
  - your name <example@domain.com>

description: your collection description
license_file: LICENSE

# TO-DO: update the tags based on your content type
tags: ["linux", "tools"]
# TO-DO: maintain this list to reflect the collection's dependencies
dependencies:
  "ansible.utils": "*" # note: "*" selects the latest version available

repository: http://example.com/repository
documentation: http://docs.example.com
homepage: http://example.com
issues: http://example.com/issue/tracker

# A list of file glob-like patterns used to filter any files or directories that should not be included in the build
# artifact. A pattern is matched from the relative path of the file or directory of the collection directory. This
# uses 'fnmatch' to match the files or directories. Some directories and files like 'galaxy.yml', '*.pyc', '*.retry',
# and '.git' are always filtered. Mutually exclusive with 'manifest'
build_ignore:
  - .gitignore
  - changelogs/.plugin-cache.yaml
# A dict controlling use of manifest directives used in building the collection artifact. The key 'directives' is a
# list of MANIFEST.in style
# L(directives,https://packaging.python.org/en/latest/guides/using-manifest-in/#manifest-in-commands). The key
# 'omit_default_directives' is a boolean that controls whether the default directives are used. Mutually exclusive
# with 'build_ignore'
# manifest: null
